<?php
/**
 * Reader class using the DOM XML extension
 *
 * @package     tgcRSS
 * @subpackage  Reader
 * @author      Carsten Lucke <luckec@tool-garage.de>
 * @copyright   Carsten Lucke <luckec@tool-garage.de>
 */
 
/**
 * Required Iterator
 */
require_once 'tgc/tgcSPL/DomNodeListIterator.php';
 
/**
 * Required Iterator
 */
require_once 'tgc/tgcSPL/DomNodeFilter.php';
 
/**
 * Reader class using the DOM XML extension
 *
 * @package     tgcRSS
 * @subpackage  Reader
 * @access      public
 * @version     0.1.0
 * @author      Carsten Lucke <luckec@tool-garage.de>
 * @todo        implement the needed methods
 */
class tgcRSS_Reader_Dom extends tgcRSS_Reader
{
   /**
    * Constructor
    * 
    * @param    string  $xml    xml
    * @throws   tgcRSSException
    */
    public function __construct($xml)
    {
        parent::__construct($xml);
        
        $this->driver = new DomDocument;
        $this->driver->preserveWhiteSpace = false;
        $this->driver->loadXML($xml);
    }
    
   /**
    * Returns the information about the RSS-feed's channel
    * 
    * @return   tgcRSS_Channel  channel-object
    * @throws   tgcRSSException
    */
    public function getChannelInfo()
    {
        $nodeList = $this->_extractData('channel');
        $this->_toArray($nodeList, $this->channel);
        return new tgcRSS_Channel($this->channel['channel']);
    }
    
   /**
    * Returns the information of the RSS-feed's image-tag
    * 
    * @return   tgcRSS_Image    image-object
    * @throws   tgcRSSException
    */
    public function getImageInfo()
    {
        $nodeList = $this->_extractData('image');
        $this->_toArray($nodeList, $this->image);
        return new tgcRSS_Image(
            isset($this->image['image']) ? $this->image['image'] : array()
        );
    }
    
   /**
    * Returns the information about the RSS-feed's item(s)
    * 
    * @return   array   array containing tgcRSS_Item objects
    * @throws   tgcRSSException
    */
    public function getItems()
    {
        $nodeList = $this->_extractData('item');
        $this->_toArray($nodeList, $this->items);
        $items = array();
        foreach ($this->items['item'] as $item) {
            $items[] = new tgcRSS_Item($item);
        }
        return $items;
    }
    
   /**
    * Returns the information of the RSS-feed's textinput-tag
    * 
    * @return   tgcRSS_Textinput    image-object
    * @throws   tgcRSSException
    */
    public function getTextinput()
    {
        $nodeList = $this->_extractData('textinput');
        $this->_toArray($nodeList, $this->textinput);
        return new tgcRSS_Textinput(
            isset($this->textinput['textinput']) ? $this->textinput['textinput'] : array()
        );
    }
    
   /**
    * Extracts the specified data from the DOMDocument.
    * 
    * @param    string  $type   desired item
    * @return   DomNodeList     list of matching nodes
    * @throws   tgcRSSException
    */
    private function _extractData($type)
    {
        $xpath = new DomXPath($this->driver);
        $query = '//' . $type;
        if (! empty($this->defaultNSUri)) {
            $ns = 'rss';
            $xpath->registerNamespace($ns, $this->defaultNSUri);
            $query = '//' . $ns . ':' . $type;
        }
        
        $nodeList = $xpath->query($query);
        if (is_null($nodeList)) {
            throw new tgcRSSException('Unable to find specified node in the RSS-feed');
        }
        return $nodeList;
    }
    
   /**
    * Converts a DomNodeList into an array.
    * 
    * @param    DomNodeList $dom    nodelist
    * @param    array       $data   data-stack (passed by reference)
    */
    private function _toArray(DomNodeList $dom, &$data)
    {
        $it = new tgcSPL_DomNodeListIterator($dom, $data);
        $fit = new tgcSPL_DomNodeFilter($it, array(XML_ELEMENT_NODE, XML_CDATA_SECTION_NODE));
        foreach ($fit as $tagName => $node) {
            // node has childnodes
            if ($this->_domNodeHasNonTextChildNodes($node)) {
                // must be an array as it has childnodes
                $aggregateP = $this->_getAggregateNames($node->parentNode);
                if (!empty($aggregateP) && in_array($tagName, $aggregateP)) {
                    $this->_toArray($node->childNodes, $data[$tagName][] = array());
                } else {
                    $this->_toArray($node->childNodes, $data[$tagName] = array());
                }
            } else {
                $aggregateP = $this->_getAggregateNames($node->parentNode);
                if (empty($aggregateP) || (!empty($aggregateP) && !in_array($tagName, $aggregateP))) {
                // no need to aggregate node-values
                    $data[$tagName] = trim($node->nodeValue);
                } else {
                // aggregation required
                    $data[$tagName][] = trim($node->nodeValue);
                }
            }
        }
    }

   /**
    * Checks if a DomNode object has other nodes than XML_TEXT_NODEs    
    *    
    * @param    object DomNode  $node   node
    * @return   boolean true if has non-text childnodes, false otherwise
    */   
    private static function _domNodeHasNonTextChildNodes(DomNode $node)
    {
        foreach ($node->childNodes as $node) {
            if ($node->nodeType != XML_TEXT_NODE) {
                return true;
            }
        }
        return false;
    }

   /**
    * Checks if a DomNode object has other nodes than XML_TEXT_NODEs    
    *    
    * @param    object DomNode  $node   node
    * @return   array   names of childnodes that have to be aggregated
    */   
    private static function _getAggregateNames(DomNode $node)
    {
        $children = array();
        $aggregate = array();
        foreach ($node->childNodes as $node) {
            if (! in_array($node->nodeName, $children)) {
                array_push($children, $node->nodeName);
            } else {
                if (! in_array($node->nodeName, $aggregate)) {
                    array_push($aggregate, $node->nodeName);
                }
            }
        }
        return $aggregate;
    }
}
?>
