<?php
/**
 * Reader class using Expat XML parser
 *
 * @package     tgcRSS
 * @subpackage  Reader
 * @author      Carsten Lucke <luckec@tool-garage.de>
 * @copyright   Carsten Lucke <luckec@tool-garage.de>
 */
 
 
/**
 * Reader class using Expat XML parser
 *
 * @package     tgcRSS
 * @subpackage  Reader
 * @access      public
 * @version     0.1.0
 * @author      Carsten Lucke <luckec@tool-garage.de>
 */
class tgcRSS_Reader_Expat extends tgcRSS_Reader
{
   /**
    * xml as array
    *
    * @var  array
    */
    protected $data = null;

   /**
    * Stack for all data found during parsing
    *
    * @var array
    */
    protected $dataStack = array();

   /**
    * Stack for all generated values
    
    * @var  array
    */
    protected $valueStack  =   array();

   /**
    * Tag depth
    *
    * @var  int
    */
    protected $depth = 0;
    
   /**
    * Constructor
    * 
    * @param    string  $xml    xml
    * @throws   tgcRSSException
    */
    public function __construct($xml)
    {
        parent::__construct($xml);
        
        $this->data = null;
        $this->valueStack = array();
        $this->dataStack = array();
        $this->depth = 0;
        
        
        $this->driver = xml_parser_create();
        xml_set_object($this->driver, $this);
        xml_parser_set_option($this->driver, XML_OPTION_CASE_FOLDING, 0);
        
        xml_set_element_handler( $this->driver, 'startElementHandler', 'endElementHandler' );
        xml_set_character_data_handler($this->driver, 'cDataHandler');
        
        if (! xml_parse($this->driver, $this->xml, true) ) {
            throw new tgcRSSException('Reader-Error: XML-Error ' . xml_get_error_code($this->driver) . 
                ' at line ' . xml_get_current_line_number($this->driver));
        } 
        
        xml_parser_free($this->driver);
    }
    
   /**
    * Expat element start-handler.
    * 
    * Inspired by PEAR::XML_Serializer (http://pear.php.net).
    * 
    * @param    resource    $parser     parser
    * @param    string      $element    element-name
    * @param    array       $atts       tag attributes
    */
    protected function startElementHandler($parser, $element, $atts)
    {
        ++$this->depth;
        $this->dataStack[$this->depth] = null;
        
        $val = array(
            'name'         => $element,
            'value'        => null,
            'type'         => 'string',
            'childrenKeys' => array(),
            'aggregKeys'   => array()
        );
        
        array_push($this->valueStack, $val);
    }
    
   /**
    * Expat element end-handler.
    * 
    * Inspired by PEAR::XML_Serializer (http://pear.php.net).
    * 
    * @param    resource    $parser     parser
    * @param    string      $element    element-name
    */
    protected function endElementHandler($parser, $element)
    {
        $value = array_pop($this->valueStack);
        $data  = trim($this->dataStack[$this->depth]);
        
        // adjust type of the value
        switch(strtolower($value['type'])) {
            // unserialize an array
            case 'array':
                if ($data !== '') {
                    $value['children']['_content'] = $data;
                }
                if (isset($value['children'])) {
                    $value['value'] = $value['children'];
                } else {
                    $value['value'] = array();
                }
                break;

            // unserialize a scalar value
            default:
                $value['value'] = $data;
                break;
        }
        
        $parent = array_pop($this->valueStack);
        if ($parent === null) {
            $this->data = &$value['value'];
        } else {
            // parent has to be an array
            if (!isset($parent['children']) || !is_array($parent['children'])) {
                $parent['children'] = array();
                $parent['type'] = 'array';
            }

            if (!empty($value['name'])) {
                // there already has been a tag with this name
                if (in_array($value['name'], $parent['childrenKeys'])) {
                    // no aggregate has been created for this tag
                    if (!in_array($value['name'], $parent['aggregKeys'])) {
                        $parent['children'][$value['name']] = array($parent['children'][$value['name']]);
                        array_push($parent['aggregKeys'], $value['name']);
                    }
                    array_push($parent['children'][$value['name']], $value['value']);
                } else {
                    $parent['children'][$value['name']] = &$value['value'];
                    array_push($parent['childrenKeys'], $value['name']);
                }
            } else {
                array_push($parent['children'],$value['value']);
            }
            array_push($this->valueStack, $parent);
            
            switch (strtolower($element)) {
                case 'channel':
                    $this->channel = $value['value'];
                    break;
                case 'item':
                    $this->items[] = $value['value'];
                    break;
                case 'image':
                    $this->image = $value['value'];
                    break;
                case 'textinput':
                    $this->textinput = $value['value'];
                    break;
            }
        }

        --$this->depth;
    }
    
   /**
    * Expat element end-handler.
    * 
    * Inspired by PEAR::XML_Serializer (http://pear.php.net).
    * 
    * @param    resource    $parser     parser
    * @param    string      $data       element's CDATA
    */
    protected function cDataHandler($parser, $data)
    {
        $this->dataStack[$this->depth] .= $data;
    }
    
   /**
    * Returns the information about the RSS-feed's channel
    * 
    * @return   tgcRSS_Channel  channel-object
    * @throws   tgcRSSException
    */
    public function getChannelInfo()
    {
        return new tgcRSS_Channel($this->channel);
    }
    
   /**
    * Returns the information of the RSS-feed's image-tag
    * 
    * @return   tgcRSS_Image    image-object
    * @throws   tgcRSSException
    */
    public function getImageInfo()
    {
        return new tgcRSS_Image($this->image);
    }
    
   /**
    * Returns the information about the RSS-feed's item(s)
    * 
    * @return   array   array containing tgcRSS_Item objects
    * @throws   tgcRSSException
    */
    public function getItems()
    {
        $items = array();
        foreach ($this->items as $item) {
            $items[] = new tgcRSS_Item($item);
        }
        return $items;
    }
    
   /**
    * Returns the information of the RSS-feed's textinput-tag
    * 
    * @return   tgcRSS_Textinput    image-object
    * @throws   tgcRSSException
    */
    public function getTextinput()
    {
        return new tgcRSS_Channel($this->textinput);
    }
}
?>
